# -*- coding: utf-8 -*-

from chameleon.core import types
from chameleon.core import config
from chameleon.core import utils

marker = ()

class Assign(object):
    """
    >>> from chameleon.core import testing

    We'll define some values for use in the tests.

    >>> _out, _write, stream = testing.setup_stream()
    >>> one = types.value("1")
    >>> bad_float = types.value("float('abc')")
    >>> abc = types.value("'abc'")
    >>> ghi = types.value("'ghi'")
    >>> exclamation = types.value("'!'")
        
    Simple value assignment:
    
    >>> assign = Assign(one)
    >>> assign.begin(stream, 'a')
    >>> exec stream.getvalue()
    >>> a == 1
    True
    >>> assign.end(stream)
    
    Try-except parts (bad, good):
    
    >>> assign = Assign(types.parts((bad_float, one)))
    >>> assign.begin(stream, 'b')
    >>> exec stream.getvalue()
    >>> b == 1
    True
    >>> assign.end(stream)
    
    Try-except parts (good, bad):
    
    >>> assign = Assign(types.parts((one, bad_float)))
    >>> assign.begin(stream, 'b')
    >>> exec stream.getvalue()
    >>> b == 1
    True
    >>> assign.end(stream)
    
    Join:

    >>> assign = Assign(types.join((abc, ghi)))
    >>> assign.begin(stream, 'b')
    >>> exec stream.getvalue()
    >>> b == 'abcghi'
    True
    >>> assign.end(stream)

    Single join:

    >>> assign = Assign(types.join((abc, )))
    >>> assign.begin(stream, 'b')
    >>> exec stream.getvalue()
    >>> b == 'abc'
    True
    >>> assign.end(stream)

    Join with try-except parts:
    
    >>> assign = Assign(types.join((types.parts((bad_float, abc, ghi)), ghi)))
    >>> assign.begin(stream, 'b')
    >>> exec stream.getvalue()
    >>> b == 'abcghi'
    True
    >>> assign.end(stream)
    """

    def __init__(self, parts, variable=None):
        if not isinstance(parts, types.parts):
            parts = types.parts((parts,))

        self.parts = parts
        self.variable = variable
        
    def begin(self, stream, variable=None):
        """First n - 1 expressions must be try-except wrapped."""

        variable = variable or self.variable
        if isinstance(variable, tuple):
            variable = ", ".join(variable)
            
        for value in self.parts[:-1]:
            stream.write("try:")
            stream.indent()

            self._assign(variable, value, stream)
            
            stream.outdent()
            stream.write("except (%s), e:" % ", ".join(
                exc.__name__ for exc in self.parts.exceptions))
            stream.indent()

        value = self.parts[-1]
        self._assign(variable, value, stream)
        
        stream.outdent(len(self.parts)-1)

    def _assign(self, variable, value, stream):
        stream.annotate(value)
        symbols = stream.symbols.as_dict()
        variable = variable % symbols

        if value.symbol_mapping:
            stream.symbol_mapping.update(value.symbol_mapping)

        if isinstance(value, types.template):
            value = types.value(value % symbols)
        if isinstance(value, types.value):
            stream.write("%s = %s" % (variable, value))
        elif isinstance(value, types.join):
            parts = []
            _v_count = 0

            for part in value:
                if isinstance(part, types.expression):
                    stream.symbol_mapping.update(part.symbol_mapping)
                if isinstance(part, types.template):
                    part = types.value(part % symbols)
                if isinstance(part, (types.parts, types.join)):
                    _v = stream.save()
                    assign = Assign(part, _v)
                    assign.begin(stream)
                    assign.end(stream)
                    _v_count +=1
                    parts.append(_v)
                elif isinstance(part, types.value):
                    parts.append(part)
                elif isinstance(part, unicode):
                    if stream.encoding:
                        parts.append(repr(part.encode(stream.encoding)))
                    else:
                        parts.append(repr(part))
                elif isinstance(part, str):
                    parts.append(repr(part))
                else:
                    raise ValueError("Not able to handle %s" % type(part))

            if len(parts) == 1:
                stream.write("%s = %s" % (variable, "".join(parts)))
            else:
                format = "%s"*len(parts)
                stream.write("%s = '%s' %% (%s)" % (variable, format, ",".join(parts)))

            for i in range(_v_count):
                stream.restore()
        else:
            raise TypeError("Can't assign value of type %s" % type(value))
        
    def end(self, stream):
        pass

class Define(object):
    """
    >>> from chameleon.core import testing

    Variable scope:

    >>> _out, _write, stream = testing.setup_stream()
    >>> define = Define("a", testing.pyexp("b"))
    >>> b = object()
    >>> define.begin(stream)
    >>> exec stream.getvalue()
    >>> a is b
    True
    >>> del a
    >>> define.end(stream)
    >>> exec stream.getvalue()
    >>> a
    Traceback (most recent call last):
        ...
    NameError: name 'a' is not defined
    >>> b is not None
    True

    Multiple defines:

    >>> _out, _write, stream = testing.setup_stream()
    >>> define1 = Define("a", testing.pyexp("b"))
    >>> define2 = Define("c", testing.pyexp("d"))
    >>> d = object()
    >>> define1.begin(stream)
    >>> define2.begin(stream)
    >>> exec stream.getvalue()
    >>> a is b and c is d
    True
    >>> define2.end(stream)
    >>> define1.end(stream)
    >>> del a; del c
    >>> exec stream.getvalue()
    >>> a
    Traceback (most recent call last):
        ...
    NameError: name 'a' is not defined
    >>> c
    Traceback (most recent call last):
        ...
    NameError: name 'c' is not defined
    >>> b is not None and d is not None
    True

    Redefining a variable which is in scope:
    
    >>> _out, _write, stream = testing.setup_stream()
    >>> define1 = Define("a", testing.pyexp("b"))
    >>> define2 = Define("a", testing.pyexp("c"))
    >>> b = object()
    >>> c = object()
    >>> define1.begin(stream)
    >>> define2.begin(stream)
    >>> exec stream.getvalue()
    >>> a is c
    True
    >>> define2.end(stream)
    >>> define1.end(stream)
    >>> del a
    >>> exec stream.getvalue()
    >>> a
    Traceback (most recent call last):
        ...
    NameError: name 'a' is not defined
    
    Tuple assignments:

    >>> _out, _write, stream = testing.setup_stream()
    >>> define = Define(types.declaration(('e', 'f')), testing.pyexp("[1, 2]"))
    >>> define.begin(stream)
    >>> exec stream.getvalue()
    >>> e == 1 and f == 2
    True
    >>> define.end(stream)

    Verify scope is preserved on tuple assignment:

    >>> _out, _write, stream = testing.setup_stream()
    >>> e = None; f = None
    >>> stream.scope[-1].add('e'); stream.scope[-1].add('f')
    >>> stream.scope.append(set())
    >>> define.begin(stream)
    >>> define.end(stream)
    >>> exec stream.getvalue()
    >>> e is None and f is None
    True

    Using semicolons in expressions within a define:

    >>> _out, _write, stream = testing.setup_stream()
    >>> define = Define("a", testing.pyexp("';'"))
    >>> define.begin(stream)
    >>> exec stream.getvalue()
    >>> a
    ';'
    >>> define.end(stream)

    Scope:

    >>> _out, _write, stream = testing.setup_stream()
    >>> a = 1
    >>> stream.scope[-1].add('a')
    >>> stream.scope.append(set())
    >>> define = Define("a", testing.pyexp("2"))
    >>> define.begin(stream)
    >>> define.end(stream)
    >>> exec stream.getvalue()
    >>> a
    1
    """

    assign = None
    
    def __init__(self, declaration, expression=None, dictionary=None):
        if not isinstance(declaration, types.declaration):
            declaration = types.declaration((declaration,))

        if len(declaration) == 1:
            variable = declaration[0]
        else:
            variable = u"(%s,)" % ", ".join(declaration)

        if declaration.global_scope and dictionary is not None:
           variable = "%s['%s'] = %s" % (dictionary, variable, variable)

        if expression is not None:
            self.assign = Assign(expression, variable)

        self.declaration = declaration
        self.dictionary = dictionary

    def begin(self, stream):
        if self.declaration.global_scope:
            # if the declaration belongs to a global scope, remove this
            # symbol from previous scopes
            for scope in stream.scope[1:]:
                for variable in self.declaration:
                    if variable in scope:
                        scope.remove(variable)
                stream.scope[1].add(variable)
        else:
            # save local variables already in in scope
            for var in self.declaration:

                # if we didn't set the variable in this scope already
                if var not in stream.scope[-1]:

                    # we'll check if it's set in one of the older
                    # scopes, e.g. if a backup is at all required
                    for scope in stream.scope[1:-1]:
                        if var in scope:
                            # in which case we back it up to a custom variable name
                            stream.write('%s%d = %s' % (
                                stream.symbols.tmp+var, stream.indentation, var))

                    stream.scope[-1].add(var)

        if self.assign is not None:
            self.assign.begin(stream)

    def end(self, stream):
        if self.assign is not None:
            self.assign.end(stream)

        if not self.declaration.global_scope:
            # restore the variables that were previously in scope
            for var in reversed(self.declaration):
                # if we set the variable in this scope already
                if var in stream.scope[-1]:
                    # we'll check if it's set in one of the older scopes
                    for scope in stream.scope[1:-1]:
                        if var in scope:
                            # in which case we restore it
                            stream.write('%s = %s%d' % (
                                var, stream.symbols.tmp+var, stream.indentation))
                            break
                    else:
                        stream.write("del %s" % var)
                    stream.scope[-1].remove(var)
                            
class Condition(object):
    """
    >>> from chameleon.core import testing, etree

    Unlimited scope:

    >>> _out, _write, stream = testing.setup_stream()
    >>> _validate = utils.validate
    >>> true = Condition(testing.pyexp("True"))
    >>> false = Condition(testing.pyexp("False"))
    >>> true.begin(stream)
    >>> stream.write("print 'Hello'")
    >>> true.end(stream)
    >>> false.begin(stream)
    >>> stream.write("print 'Universe!'")
    >>> false.end(stream)
    >>> stream.write("print 'World!'")
    >>> exec stream.getvalue()
    Hello
    World!

    Finalized limited scope:

    >>> _out, _write, stream = testing.setup_stream()
    >>> true = Condition(testing.pyexp("True"), [Write(testing.pyexp("'Hello'"))])
    >>> false = Condition(testing.pyexp("False"), [Write(testing.pyexp("'Hallo'"))])
    >>> true.begin(stream)
    >>> true.end(stream)
    >>> false.begin(stream)
    >>> false.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    'Hello'

    Open limited scope:

    >>> _out, _write, stream = testing.setup_stream()
    >>> true = Condition(testing.pyexp("True"), [Tag('div')], finalize=False)
    >>> false = Condition(testing.pyexp("False"), [Tag('span')], finalize=False)
    >>> true.begin(stream)
    >>> stream.out("Hello World!")
    >>> true.end(stream)
    >>> false.begin(stream)
    >>> false.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    '<div>Hello World!</div>'

    """
      
    def __init__(self, value, clauses=None, finalize=True, invert=False):
        self.assign = Assign(value)
        self.clauses = clauses
        self.finalize = finalize
        self.invert = invert

    def begin(self, stream):
        temp = stream.save()
        self.assign.begin(stream, temp)

        if self.invert:
            stream.write("if not (%s):" % temp)
        else:
            stream.write("if %s:" % temp)

        stream.indent()
        stream.write("pass")
        
        if self.clauses:
            for clause in self.clauses:
                clause.begin(stream)
            if self.finalize:
                for clause in reversed(self.clauses):
                    clause.end(stream)
                stream.restore()
            stream.outdent()
        elif self.finalize:
            stream.restore()
            
    def end(self, stream):
        if self.clauses:
            if not self.finalize:
                temp = stream.restore()
                if self.invert:
                    stream.write("if not (%s):" % temp)
                else:
                    stream.write("if %s:" % temp)
                stream.indent()
                stream.write("pass")
                for clause in reversed(self.clauses):
                    clause.end(stream)
                    stream.outdent()
        else:
            if not self.finalize:
                stream.restore()
            stream.outdent()
        self.assign.end(stream)

class Else(object):
    def __init__(self, clauses=None):
        self.clauses = clauses
        
    def begin(self, stream):
        stream.write("else:")
        stream.indent()
        stream.write("pass")
        if self.clauses:
            for clause in self.clauses:
                clause.begin(stream)
            for clause in reversed(self.clauses):
                clause.end(stream)
            stream.outdent()
        
    def end(self, stream):
        if not self.clauses:
            stream.outdent()

class Group(object):
    def __init__(self, clauses):
        self.clauses = clauses
        
    def begin(self, stream):
        for clause in self.clauses:
            clause.begin(stream)
        for clause in reversed(self.clauses):
            clause.end(stream)

    def end(self, stream):
        pass

class Visit(object):
    def __init__(self, node):
        self.node = node
        
    def begin(self, stream):
        self.node.visit()

    def end(self, stream):
        pass

class Tag(object):
    """
    >>> from chameleon.core import testing

    Dynamic attribute:

    >>> _out, _write, stream = testing.setup_stream()
    >>> _default = object()
    
    >>> tag = Tag('div', dict(alt=testing.pyexp(repr('Hello World!'))))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    '<div alt="Hello World!">Hello Universe!</div>'

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> tag = Tag('div', dict(alt=testing.pyexp(repr('Hello World!'))))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'<div alt="Hello World!">Hello Universe!</div>'

    Cancelling attribute via false value:

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> tag = Tag('div', dict(alt=testing.pyexp(None)))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    '<div>Hello Universe!</div>'

    Escaping of <, >, & and double quotes

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> tag = Tag('div', dict(alt=testing.pyexp(repr('<>&"'))))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'<div alt="&lt;&gt;&amp;&quot;">Hello Universe!</div>'

    Attribute default value is available under the `default` symbol.

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> tag = Tag('div', dict(alt=testing.pyexp('default')),
    ...     defaults=dict(alt='Hello World!'))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'<div alt="Hello World!">Hello Universe!</div>'

    Verify that unicode data is handled correctly.

    >>> _out, _write, stream = testing.setup_stream()
    >>> tag = Tag('div', dict(
    ...     alt=testing.pyexp(repr(unicode('La Pe\xc3\xb1a', 'utf-8')))))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> 'Hello' in _out.getvalue()
    True

    Dictionary attributes:

    >>> _out, _write, stream = testing.setup_stream()
    >>> tag = Tag('div', expression=testing.pyexp(repr({'alt': 'Hello World!'})))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    '<div alt="Hello World!">Hello Universe!</div>'
    
    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> tag = Tag('div', expression=testing.pyexp(repr({'alt': 'Hello World!'})))
    >>> tag.begin(stream)
    >>> stream.out('Hello Universe!')
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'<div alt="Hello World!">Hello Universe!</div>'

    Self-closing tag:

    >>> _out, _write, stream = testing.setup_stream()
    >>> tag = Tag('br', {}, True)
    >>> tag.begin(stream)
    >>> tag.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    '<br />'
    """

    def __init__(self, tag, attributes=None, selfclosing=False,
                 expression=None, cdata=False, defaults={}):
        """Render tag.

        Note that ``attributes`` must be rendered in order, except if
        they're replaced by the attributes dictionary in the evaluated
        ``expression``.
        """
        
        self.tag = tag
        self.defaults = defaults
        self.selfclosing = selfclosing
        self.attributes = attributes or {}
        self.expression = expression and Assign(expression)
        self.cdata = cdata
        
    def begin(self, stream):
        if self.cdata:
            stream.out('<![CDATA['); return

        stream.out('<%s' % self.tag)

        temp = stream.save()
        temp2 = stream.save()
        
        if self.expression:
            self.expression.begin(stream, stream.symbols.tmp)
            # loop over all attributes
            stream.write("for %s, %s in %s.items():" % \
                         (temp, temp2, stream.symbols.tmp))
            stream.indent()

            # only include attribute if expression is not None
            stream.write("if %s is not None:" % temp2)
            stream.indent()

            # if an encoding is specified, we need to check
            # whether we're dealing with unicode strings or not,
            # before writing out the attribute
            stream.ensure_unicode(temp2)

            # escape expression
            stream.escape(temp2, escape_double_quote=True)

            # write out attribute
            stream.out(types.value(
                "' %%s=\"%%s\"' %% (%s, %s)" % (temp, temp2)))

            stream.outdent()
            stream.outdent()

        for attribute, value in self.attributes.items():
            if isinstance(value, types.expression):
                # as an optimization, we only define the `default` symbol,
                # if it's present in the evaluation value (as string
                # representation).
                if stream.symbols.default in str(value):
                    default = self.defaults.get(attribute, marker)
                else:
                    default = None

                if default is marker:
                    stream.write("%s = %s" % (
                        stream.symbols.default,
                        stream.symbols.default_marker_symbol))
                elif default is not None:
                    stream.write("%s = %s" % (stream.symbols.default, repr(default)))

                assign = Assign(value)
                assign.begin(stream, temp)

                if default is not None:
                    stream.write("%s = None" % stream.symbols.default)

                # verify that attribute value is not the default marker
                stream.write("if %s is %s:" % (
                    temp, stream.symbols.default_marker_symbol))
                default_string = self.defaults.get(attribute)
                stream.indent()
                stream.write("%s = %s" % (
                    temp, repr(default_string)))
                stream.outdent()

                stream.write("if %s is None or %s is False:" % (temp, temp))
                stream.indent()
                stream.write("pass")
                stream.outdent()
                stream.write("else:")
                stream.indent()

                # if an encoding is specified, we need to check
                # whether we're dealing with unicode strings or not,
                # before writing out the attribute
                stream.ensure_unicode(temp)

                # escape expression
                stream.escape(temp, escape_double_quote=True)

                # print attribute
                stream.out(types.value(
                    "' %s=\"'+%s+'\"'" % (
                    attribute, temp)))

                stream.outdent()
                assign.end(stream)
            else:
                # escape expression
                value = utils.escape(value, '"', 'ascii')

                # if there are dynamic expressions, we only want to write
                # out static attributes if they're not in the dynamic
                # expression dictionary
                if self.expression:
                    stream.write("if '%s' not in %s:" % (
                        attribute, stream.symbols.tmp))
                    stream.indent()

                stream.out(' %s="%s"' % (attribute, value))

                if self.expression:
                    stream.outdent()

        stream.restore()
        stream.restore()

        if self.selfclosing:
            stream.out(" />")
        else:
            stream.out(">")

    def end(self, stream):
        if self.cdata:
            stream.out(']]>'); return
            
        if not self.selfclosing:
            stream.out('</%s>' % self.tag)

class Repeat(object):
    """
    >>> from chameleon.core import testing

    We need to set up the repeat object.

    >>> from chameleon.core import utils
    >>> repeat = utils.repeatdict()

    Simple repeat loop and repeat data structure:

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("i",)), testing.pyexp("range(5)"))
    >>> _repeat.begin(stream)
    >>> stream.write("r = repeat['i']")
    >>> stream.write(
    ...     "print (i, r.index, r.letter(), r.Letter(), r.roman(), r.Roman(), r.start, r.end, r.number(), r.odd(), r.even())")
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()
    (0, 0, 'a', 'A', 'i', 'I', True, False, 1, False, True)
    (1, 1, 'b', 'B', 'ii', 'II', False, False, 2, True, False)
    (2, 2, 'c', 'C', 'iii', 'III', False, False, 3, False, True)
    (3, 3, 'd', 'D', 'iv', 'IV', False, False, 4, True, False)
    (4, 4, 'e', 'E', 'v', 'V', False, True, 5, False, True)
    >>> _repeat.end(stream)

    A repeat over an empty set.

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("j",)), testing.pyexp("range(0)"))
    >>> _repeat.begin(stream)
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()

    A tuple repeat over an empty set.

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("j", "k")),
    ...      testing.pyexp("range(0)"), repeatdict=False)
    >>> _repeat.begin(stream)
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()

    A tuple repeat

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("l", "m")),
    ...      testing.pyexp("zip(range(3), range(3))"), repeatdict=False)
    >>> _repeat.begin(stream)
    >>> stream.write("print l, m")
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()
    0 0
    1 1
    2 2
    
    A repeat over an iterable which has no length, renders the generator.

    >>> class iterator(object):
    ...     def __iter__(self):
    ...         yield 1; yield 2; yield 3
    
    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("i",)), testing.pyexp("iterator()"))
    >>> _repeat.begin(stream)
    >>> stream.write("r = repeat['i']")
    >>> stream.write(
    ...     "print (i, r.index, r.start, r.end, r.number(), r.odd(), r.even())")
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()
    (1, 0, True, False, 1, False, True)
    (2, 1, False, False, 2, True, False)
    (3, 2, False, True, 3, False, True)
    >>> _repeat.end(stream)

    A repeat over a non-iterable raises an exception.

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("j",)), testing.pyexp("object()"))
    >>> _repeat.begin(stream)
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()
    Traceback (most recent call last):
     ...
    TypeError: 'object' object is not iterable

    Repeating over ``None`` is equal to repeating over the empty tuple.

    >>> _out, _write, stream = testing.setup_stream()
    >>> _repeat = Repeat(types.declaration(("j",)), testing.pyexp("None"))
    >>> _repeat.begin(stream)
    >>> _repeat.end(stream)
    >>> exec stream.getvalue()

    Simple for loop:
  
    >>> _out, _write, stream = testing.setup_stream()
    >>> _for = Repeat(types.declaration(("i",)),
    ...     testing.pyexp("range(3)"), repeatdict=False)
    >>> _for.begin(stream)
    >>> stream.write("print i")
    >>> _for.end(stream)
    >>> exec stream.getvalue()
    0
    1
    2
    >>> _for.end(stream)

    """

    def __init__(self, declaration, value, scope=(), repeatdict=True, newline=True):
        if len(declaration) > 1:
            assert repeatdict is False

        self.declaration = declaration            
        self.define = Define(declaration)
        self.assign = Assign(value)
        self.repeatdict = repeatdict
        self.newline = newline
        
    def begin(self, stream):
        # initialize variable scope
        self.define.begin(stream)

        # assign iterator
        iterator = stream.save()
        length = stream.save()
        self.assign.begin(stream, iterator)

        # initialize variables
        stream.write("%s = %s" % (
            ", ".join(self.declaration),
            ", ".join((repr(None),)*len(self.declaration))))
        
        if self.repeatdict:
            variable = self.declaration[0]
            assert ',' not in variable
            stream.write("%s, %s = repeat.insert('%s', %s)" % (
                iterator, length, variable, iterator))
            stream.write("for %s in %s:" % (
                variable, iterator))
        else:
            stream.write("%s = tuple(%s)" % (iterator, iterator))
            stream.write("%s = len(%s)" % (length, iterator))
            stream.write("for %s in %s:" % (
                ", ".join(self.declaration), iterator))
        stream.indent()
        stream.write("%s = %s - 1" % (length, length))

    def end(self, stream):
        length = stream.restore()
        iterator = stream.restore()

        stream.write("if %s == 0:" % length)
        stream.indent()
        stream.write("break")
        stream.outdent()

        if self.newline:
            stream.out('\n')
        else:
            stream.out(' ')
        stream.outdent()

        self.assign.end(stream)
        self.define.end(stream)

class Write(object):
    """
    >>> from chameleon.core import testing, etree

    Basic write:

    >>> _out, _write, stream = testing.setup_stream()
    >>> _validate = utils.validate
    >>> write = Write(testing.pyexp("'New York'"))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    'New York'

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> write = Write(testing.pyexp("'New York'"))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'New York'

    Try-except parts:

    >>> _out, _write, stream = testing.setup_stream()
    >>> write = Write(testing.pyexp('undefined', '"New Delhi"'))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    'New Delhi'

    Unicode:

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> write = Write(types.value(repr('La Pe\xc3\xb1a'.decode('utf-8'))))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> val = _out.getvalue()
    >>> val == u'La Pe\xf1a'
    True
    >>> isinstance(val, unicode)
    True

    Object that offers an ``__html__`` method:

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> class Markup(unicode):
    ...     def __html__(self): return self
    >>> data = Markup("one & two")
    >>> write = Write(
    ...    types.escape(types.value('data')))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    u'one & two'
    """

    assign = None
    
    def __init__(self, value, defer=False):
        self.assign = Assign(value)
        self.structure = not isinstance(value, types.escape)
        self.defer = defer
        
    def begin(self, stream):
        if not self.defer:
            self.write(stream)
            
    def end(self, stream):
        if self.defer:
            self.write(stream)
    
    def write(self, stream):
        temp = stream.save()
        symbols = stream.symbols.as_dict()

        def write(template):
            stream.write(template % symbols)

        self.assign.begin(stream, temp)
        expr = temp

        stream.write("%s = %s" % (stream.symbols.tmp, expr))
        write("if %(tmp)s.__class__ not in (str, unicode, int, float) and hasattr(%(tmp)s, '__html__'):")
        stream.indent()
        stream.out(types.value("%s.__html__()" % symbols['tmp']))
        stream.outdent()
        
        write("elif %(tmp)s is not None:")
        stream.indent()

        stream.ensure_unicode(stream.symbols.tmp)

        # escape non-structural values
        if not self.structure:
            stream.escape(stream.symbols.tmp)

        stream.out(types.value(symbols['tmp']))
        stream.outdent()

        # validate XML if enabled
        if config.VALIDATION:
            try:
                _et = utils.import_elementtree()
            except ImportError:
                raise ImportError(
                    "ElementTree (required when XML validation is enabled).")

            stream.symbol_mapping[stream.symbols.validate] = utils.validate
            write("%(validate)s(%(tmp)s)")

        self.assign.end(stream)
        stream.restore()

class UnicodeWrite(Write):
    """
    >>> from chameleon.core import testing, etree

    Basic write:

    >>> _out, _write, stream = testing.setup_stream()
    >>> _validate = utils.validate
    >>> write = UnicodeWrite(types.value("'New York'"))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    'New York'

    Unicode:

    >>> _out, _write, stream = testing.setup_stream('utf-8')
    >>> write = UnicodeWrite(types.value(repr(unicode('La Pe\xc3\xb1a', 'utf-8'))))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> val = _out.getvalue()
    >>> val == u'La Pe\xf1a'
    True
    >>> type(val) == unicode
    True

    Invalid:

    >>> _out, _write, stream = testing.setup_stream()
    >>> write = UnicodeWrite(types.value("None"))
    >>> write.begin(stream)
    >>> write.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    ''
    """

    def write(self, stream):
        temp = stream.save()
        self.assign.begin(stream, temp)
        stream.out(types.value(temp))
        self.assign.end(stream)
        stream.restore()

class Out(object):
    """
    >>> from chameleon.core import testing
      
    >>> _out, _write, stream = testing.setup_stream()
    >>> out = Out('Hello World!')
    >>> out.begin(stream)
    >>> out.end(stream)
    >>> exec stream.getvalue()
    >>> _out.getvalue()
    'Hello World!'
    """
    
    def __init__(self, string, defer=False):
        self.string = string
        self.defer = defer

    def begin(self, stream):
        if not self.defer:
            stream.out(self.string)
        
    def end(self, stream):
        if self.defer:
            stream.out(self.string)

class Callback(object):
    """Callback method for an element visitor."""

    def __init__(self, name, visitor, args, newline):
        self.name = name
        self.visitor = visitor
        self.args = args
        self.newline = newline
        
    def begin(self, stream):
        stream.write("def %s(%s, %s, **_ignored):" % (
            self.name, stream.symbols.scope, self.args))
        stream.indent()
        stream.write("pass")
        
        # visit slot node
        self.visitor.begin(stream)
        self.visitor.end(stream)

        if self.newline:
            stream.out('\n')
            
        stream.outdent()
        
    def end(self, stream):
        pass

class Slot(object):
    """Call slot, updating """

    def __init__(self, expression, scope_args):
        self.expression = expression
        self.scope_args = scope_args

    def begin(self, stream):
        stream.write("%s.update(dict(%s))" % (
            stream.symbols.scope,
            ", ".join("%s=%s" % (arg, arg) for arg in self.scope_args)))

        value = self.expression
        if isinstance(value, types.template):
            symbols = stream.symbols.as_dict()
            value = types.value(value % symbols)

        stream.write(value)

    def end(self, stream):
        pass

class Macro(object):
    """Use macro."""

    def __init__(self, slots, args, extend=False, extend_except=None, label=None):
        self.assign = Assign(slots)
        self.extend = extend
        self.extend_except = extend_except
        self.args = args
        self.label = label
        
    def begin(self, stream):
        self.assign.begin(stream, stream.symbols.tmp)

        if self.extend:
            stream.write("for _name in %s:" % stream.symbols.slots)
            stream.indent()
            stream.write("if _name not in %s and _name not in (%s,):" % (
                stream.symbols.tmp, ", ".join(map(repr, self.extend_except))))
            stream.indent()
            stream.write("%s[_name] = %s[_name]" % (
                stream.symbols.tmp, stream.symbols.slots))
            stream.outdent()
            stream.outdent()

        stream.annotate(self.label)
        stream.write("%s.render(%s, %s)" % (
            stream.symbols.metal, stream.symbols.tmp, self.args))

    def end(self, stream):
        self.assign.end(stream)
        
class Method(object):
    """
    >>> from chameleon.core import testing
      
    >>> _out, _write, stream = testing.setup_stream()
    >>> econtext = {}
    >>> method = Method('test', ('a', 'b', 'c'))
    >>> method.begin(stream)
    >>> stream.write('print a, b, c')
    >>> method.end(stream)
    >>> exec stream.getvalue()
    >>> test(1, 2, 3)
    1 2 3
      
    """

    ret = None
    
    def __init__(self, name, args, ret=None, decorators=(), dictionary=None):
        self.name = name
        self.args = args
        self.decorators = decorators
        self.dictionary = dictionary
        
        if ret is not None:
            self.ret = Assign(ret, '_ret')

    def begin(self, stream):
        for decorator in self.decorators:
            stream.write("@%s" % decorator)
        stream.write('def %s(%s):' % (self.name, ", ".join(self.args)))
        stream.indent()

    def end(self, stream):
        if self.ret is not None:
            self.ret.begin(stream)
            stream.write('return _ret')
            self.ret.end(stream)
        
        stream.outdent()

        if self.dictionary is not None:
            assign = Assign(
                types.value(self.name), "%s['%s']" % \
                (self.dictionary, self.name))
            assign.begin(stream)
            assign.end(stream)



import os
import sys
import inspect
import doctypes
import tempfile
import utils
import i18n
import config
import filecache
import translation

try:
    from hashlib import sha1 as sha
except ImportError:
    from sha import sha

class Template(object):
    """Base template class.

    Note: template language implementations will usually make a
    subclass available for convenience.

    A language parser must be provided as ``parser``. The template
    ``body`` parameter must be a string.

    Most language parsers support two modes of operation: XML and text
    mode. This can be configured using the ``format`` parameter
    (provide either 'text' or 'xml').

    The default string type is unicode. Pass a value for ``encoding``
    if you need to allow strings that are not unicode or plain
    ASCII. Note that the output will always be unicode.

    To enable default attribute prefix rendering, use the
    ``omit_default_prefix`` parameter (required for certain legacy
    applications).
    """

    format = 'xml'
    filename = '<string>'
    version = 8
    registry = None
    explicit_doctype = None
    encoding = None
    translate = staticmethod(i18n.fast_translate)

    def __init__(self, body, parser, format=None, doctype=None,
                 encoding=None, omit_default_prefix=True):
        if encoding is not None:
            self.encoding = encoding

        if self.registry is None:
            # in debug-mode we initialize a filecache to a named
            # temporary file; this makes step-debugging through
            # generated template code possible
            if config.DEBUG_MODE:
                self.registry = filecache.TemplateCache(
                    tempfile.NamedTemporaryFile('w').name, 0)
            else:
                self.registry = filecache.TemplateRegistry()

        if format is not None:
            self.format = format

        if doctype is not None:            
            self.explicit_doctype = doctype

        self.parser = parser
        self.body = body
        self.omit_default_prefix = omit_default_prefix
        self.signature = sha(";".join(map(str, (
            type(parser).__name__, format,
            encoding, omit_default_prefix,
            self.explicit_doctype)))).hexdigest()

    def compiler(self, *args):
        self.parse()
        return self.__dict__['compiler'](*args)

    def __repr__(self):
        return u"<%s %s>" % (self.__class__.__name__, self.filename)

    def __call__(self, **kwargs):
        return self.render(**kwargs)

    def write(self, body):
        self.__dict__['body'] = body
        self.__dict__.pop('compiler', None)
        if config.EAGER_PARSING:
            self.parse()

    def parse(self):
        body = self.__dict__['body']

        # if the body is trivial, we don't try to compile it
        if None in (body, self.parser):
            return

        parse_method = getattr(self.parser, 'parse_%s' % self.format)

        try:
            tree = parse_method(body)
        except:
            utils.raise_template_exception(
                "", repr(self), {}, sys.exc_info())

        # it's not clear from the tree if an XML declaration was
        # present in the document source; the following is a
        # work-around to ensure that output matches input
        if '<?xml ' in body:
            xml_declaration = \
            """<?xml version="%s" encoding="%s" standalone="no" ?>""" % (
                tree.docinfo.xml_version, tree.docinfo.encoding)
        else:
            xml_declaration = None

        explicit_doctype = self.explicit_doctype
        if not explicit_doctype is doctypes.no_doctype and not explicit_doctype:
            explicit_doctype = self.parser.doctype

        compiler = translation.Compiler(
            tree, explicit_doctype=explicit_doctype,
            xml_declaration=xml_declaration, encoding=self.encoding,
            omit_default_prefix=self.omit_default_prefix)

        if config.RUNTIME_COMPILATION is False:
            compiler = None

        self.__dict__['compiler'] = compiler
        self.__dict__['slots']  = compiler.macros
        self.__dict__['macros'] = Macros(self.render_macro, *compiler.macros)

    @property
    def macros(self):
        try:
            return self.__dict__['macros']
        except KeyError:
            self.parse()
            return self.__dict__['macros']

    @property
    def slots(self):
        try:
            return self.__dict__['slots']
        except KeyError:
            self.parse()
            return self.__dict__['slots']

    body = property(lambda template: template.__dict__['body'], write)

    def cook_and_render(self, kwargs, slots, macro, global_scope):
        """Cook and render template.

        This method finds a render-method from the template registry
        facility, cooking one if required (by invoking the compiler).
        """

        if self.compiler is None:
            raise NotImplemented("Python compiler-support required.")

        key = macro, global_scope, self.signature
        if key not in self.registry:
            source = self.compiler(macro, global_scope)
            self.registry.add(key, source)
            if key not in self.registry:
                raise RuntimeError(
                    "Unable to add template '%s' to registry (%s)." % (
                        self.filename, type(self.registry).__name__))

        func = self.registry[key]

        econtext = rcontext = kwargs.pop("econtext", self)
        if global_scope is False:
            econtext = econtext.copy()
        if econtext is self:
            econtext = utils.econtext(kwargs)
        else:
            econtext.update(kwargs)
        econtext[config.SYMBOLS.slots] = slots
        econtext.setdefault(config.SYMBOLS.translate, self.translate)

        __traceback_info__ = (self,)
        if config.DEBUG_MODE is False:
            return func(econtext, rcontext)
        try:
            return func(econtext, rcontext)
        except:
            source = func.__dict__.get('source') or inspect.getsource(func)
            utils.raise_template_exception(
                source, repr(self), kwargs, sys.exc_info())

    def render(self, **kwargs):
        return self.cook_and_render(kwargs, utils.emptydict, None, True)

    def render_macro(self, macro, global_scope=False, slots={}, parameters={}):
        if config.STRICT_MODE and macro != '':
            extend, names = self.slots[macro]
            if extend is False:
                for name in slots:
                    if name not in names:
                        raise KeyError(name)

        return self.cook_and_render(parameters, slots, macro, global_scope)

    def render_xinclude(self, **kwargs):
        return self.render_macro("", parameters=kwargs)

class TemplateFile(Template):
    """Constructs a template object using the template language
    defined by ``parser``. Must be passed an absolute (or
    current-working-directory-relative) filename as ``filename``. If
    ``auto_reload`` is true, each time the template is rendered, it
    will be recompiled if it has been changed since the last
    rendering."""

    content_type = None
    global_registry = {}

    def __init__(self, filename, parser, format=None,  doctype=None,
                 auto_reload=config.DEBUG_MODE):
        # compute absolute filename
        self.filename = filename = os.path.abspath(
            os.path.normpath(os.path.expanduser(filename)))

        # make sure file exists
        os.lstat(filename)

        # persist template registry on disk
        if config.DISK_CACHE:
            hierarchy = sorted(
                utils.class_hierarchy(type(self)), key=utils.dotted_name)
            versions = (base.__dict__.get('version') for base in hierarchy)
            version = ".".join(map(str, filter(None, versions)))
            self.registry = filecache.TemplateCache(filename, version)

        # initialize template
        Template.__init__(self, None, parser, format=format, doctype=doctype)

        # read template (note that if we're unable to read the file,
        # we set ``auto_reload`` to true)
        if self.read() is False:
            auto_reload = True

        self.auto_reload = auto_reload
        self.global_registry.setdefault(filename, self)
        self.xincludes = XIncludes(
            self.global_registry, os.path.dirname(filename), self.clone)

    def clone(self, filename, format=None):
        cls = type(self)
        return cls(
            filename, self.parser, format=format,
            doctype=self.explicit_doctype, auto_reload=self.auto_reload)

    def _get_filename(self):
        return getattr(self, '_filename', None)

    def _set_filename(self, filename):
        self._filename = filename
        self._v_last_read = False

    filename = property(_get_filename, _set_filename)

    def read(self):
        filename = self.filename
        mtime = self.mtime()

        __traceback_info__ = filename
        fd = open(filename, "rb")
        try:
            body = fd.read(utils.XML_PREFIX_MAX_LENGTH)
        except:
            fd.close()
            raise

        content_type = utils.sniff_type(body)
        if content_type == "text/xml":
            body += fd.read()
            fd.close()
        else:
            # For HTML, we really want the file read in text mode:
            fd.close()
            fd = open(filename)
            body = fd.read()
            fd.close()

            # Look for an encoding specification in the meta tag
            match = utils.re_meta.search(body)
            if match is not None:
                content_type, encoding = match.groups()
            else:
                content_type = None
                encoding = config.DEFAULT_ENCODING
            try:
                body = unicode(body, encoding).encode(config.DEFAULT_ENCODING)
            except UnicodeDecodeError:
                body = None

        if body is not None:
            self.body = body
            self.content_type = content_type
            self._v_last_read = mtime

        # purge registry if source file is newer
        if self.registry.mtime < mtime:
            self.registry.purge()

        return body is not None

    def cook_and_render(self, args, slots, macro, global_scope):
        if self.auto_reload and self._v_last_read != self.mtime():
            if self.read() is False:
                raise RuntimeError(
                    "Unable to read body (%s)." % self.filename)

        return super(TemplateFile, self).cook_and_render(
            args, slots, macro, global_scope)

    @property
    def macros(self):
        if self.auto_reload and self._v_last_read != self.mtime():
            if self.read() is False:
                raise RuntimeError(
                    "Unable to read body (%s)." % self.filename)
        return Template.macros.fget(self)

    @property
    def slots(self):
        if self.auto_reload and self._v_last_read != self.mtime():
            if self.read() is False:
                raise RuntimeError(
                    "Unable to read body (%s)." % self.filename)
        return Template.slots.fget(self)

    def render(self, **kwargs):
        kwargs[config.SYMBOLS.xincludes] = self.xincludes
        return super(TemplateFile, self).render(**kwargs)

    def mtime(self):
        try:
            return os.path.getmtime(self.filename)
        except (IOError, OSError):
            return 0

class XIncludes(object):
    """Dynamic XInclude registry providing a ``get``-method that will
    resolve a filename to a template instance. Format must be
    explicitly provided."""
    
    def __init__(self, registry, relpath, factory):
        self.registry = registry
        self.relpath = relpath
        self.factory = factory

    def get(self, filename, format):
        if not os.path.isabs(filename):
            filename = os.path.join(self.relpath, filename)        
        template = self.registry.get(filename)
        if template is not None:
            return template
        return self.factory(filename, format=format)

class Macro(object):
    def __init__(self, render):
        self.render = render

class Macros(object):
    def __init__(self, render_macro, *names, **kwargs):
        self.render = render_macro
        self.bound_parameters = kwargs
        self.names = names

    def __getitem__(self, name):
        if name and name not in self.names:
            raise KeyError(name)
        return self.get_macro(name)

    def get_macro(self, name):
        def render(slots, **kwargs):
            kwargs.update(self.bound_parameters)
            return self.render(name, slots=slots, parameters=kwargs)
        return Macro(render)

    def bind(self, **kwargs):
        return Macros(self.render, *self.names, **kwargs)



import os
import sys
import logging
import py_compile

try:
    from hashlib import sha1 as sha
except ImportError:
    from sha import sha

try:
    import imp
except ImportError:
    imp = None

logger = logging.getLogger("Chameleon")

class TemplateRegistry(object):
    version = None
    mtime = None
    
    def __init__(self):
        self.registry = {}

    def __getitem__(self, key):
        return self.registry[key]

    def __contains__(self, key):
        if key in self.registry:
            render = self.registry[key]
            if render is None:
                self.load()
                render = self.registry.get(key)
            return render is not None
        return False

    def add(self, key, source):
        _locals = {}
        exec source in _locals
        bind = _locals['bind']
        func = bind()
        func.source = source
        self.registry[key] = func

    def clear(self):
        self.registry.clear()
        self.registry['version'] = self.version

    def load(self):
        pass
    
    def purge(self):
        self.clear()

class TemplateCache(TemplateRegistry):
    def __init__(self, filename, version):
        self.filename = filename
        self.version = version
        self.registry = {}
        try:
            self.load()
        except IOError:
            self.clear()

    def __len__(self):
        return len(self.registry) - 1

    @property
    def mtime(self):
        try:
            return os.path.getmtime(self.module_filename)
        except (IOError, OSError):
            return 0

    @property
    def module_filename(self):
        return self.filename + os.extsep + "py"

    def load(self):
        filename = self.module_filename

        try:
            if imp is not None:
                module_name = "chameleon_%s" % sha(filename).hexdigest()
                f = open(filename, 'r')
                try:
                    if module_name in sys.modules:
                        del sys.modules[module_name]
                    module = imp.load_source(module_name, filename, f)
                finally:
                    f.close()
                registry = module.registry
            else:
                _locals = {}
                execfile(filename, _locals)
                registry = _locals['registry']

            version = registry['version']
            if version != self.version:
                raise ValueError("Version mismatch: %s != %s" % (
                    version, self.version))
        except (AttributeError, ValueError, TypeError), e:
            logger.debug(
                "Error loading cache for %s (%s)." % (self.filename, str(e)))
            self.purge()
        else:
            self.registry.clear()
            self.registry.update(registry)

    def add(self, key, source):
        """Add template to module.

        We simply append the function definition (closure inside a
        ``bind`` method) and update the registry, e.g.

          >>> registry[key] = bind()

        """

        if os.path.exists(self.module_filename):
            module = open(self.module_filename, 'a')
        else:
            module = open(self.module_filename, 'w')
            self.initialize(module)
        try:
            module.write(source+'\n')
            module.write("registry[%s] = bind()\n" % repr(key))
        finally:
            module.close()

        py_compile.compile(self.module_filename)
        self.registry[key] = None

    def initialize(self, module):
        module.write("registry = dict(version=%s)\n" % repr(self.version))

    def purge(self):
        self.clear()
        
        # write empty file
        module = open(self.module_filename, 'w')
        self.initialize(module)
        module.close()
        py_compile.compile(self.module_filename)



import os
import unittest
import doctest

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def docfilesuite(func):
    def handler(cls):
        path = func(cls)
        return doctest.DocFileSuite(
            path, optionflags=OPTIONFLAGS,
            package="chameleon", encoding="utf-8")
    handler.__name__ = func.__name__
    return handler

class CoreTests(unittest.TestCase):
    def __new__(self, test):
        from chameleon.core import config
        config.DISK_CACHE = False
        config.DEBUG_MODE = True
        config.STRICT_MODE = True
        return getattr(self, test)()

    @classmethod
    @docfilesuite
    def test_template(cls):
        return os.path.join('core', 'template.txt')

    @classmethod
    @docfilesuite
    def test_codegen(cls):
        return os.path.join('core', 'codegen.txt')

    @classmethod
    @docfilesuite
    def test_translation(cls):
        return os.path.join('core', 'translation.txt')

    @classmethod
    @docfilesuite
    def test_filecache(cls):
        return os.path.join('core', 'filecache.txt')

    @classmethod
    def test_translation_module(cls):
        from chameleon.core import translation
        return doctest.DocTestSuite(translation, optionflags=OPTIONFLAGS)

    @classmethod
    def test_clauses_module(cls):
        from chameleon.core import clauses
        return doctest.DocTestSuite(clauses, optionflags=OPTIONFLAGS)

    @classmethod
    def test_parsing_module(cls):
        from chameleon.core import parsing
        return doctest.DocTestSuite(parsing, optionflags=OPTIONFLAGS)

    @classmethod
    def test_utils_module(cls):
        from chameleon.core import utils
        return doctest.DocTestSuite(utils, optionflags=OPTIONFLAGS)

class ZopePageTemplatesTests(unittest.TestCase):
    def __new__(self, test):
        return getattr(self, test)()

    @classmethod
    @docfilesuite
    def test_language(cls):
        return os.path.join('zpt', 'language.txt')

    @classmethod
    @docfilesuite
    def test_template(cls):
        return os.path.join('zpt', 'template.txt')

    @classmethod
    @docfilesuite
    def test_i18n(cls):
        return os.path.join('zpt', 'i18n.txt')

    @classmethod
    def test_language_module(cls):
        from chameleon.zpt import language
        return doctest.DocTestSuite(language, optionflags=OPTIONFLAGS)

    @classmethod
    def test_expressions_module(cls):
        from chameleon.zpt import expressions
        return doctest.DocTestSuite(expressions, optionflags=OPTIONFLAGS)
